
package Wiki::HTML;
use Mod::HTML;
@ISA = ('Mod::HTML');

%paragraph = (
              'whitespace' => \&whitespace,
              'prose' => \&prose,
              'program' => \&program,
             );

%command = 
  (
   'section' => \&Mod::HTML::section,
   'subsection' => \&Mod::HTML::section,
   'subsubsection' => \&Mod::HTML::section,

   'bulletedlist' => \&Mod::HTML::list,
   'endbulletedlist' => \&Mod::HTML::list,
   'numberedlist' => \&Mod::HTML::list,
   'endnumberedlist' => \&Mod::HTML::list,
   'item' => \&Mod::HTML::listitem,

   'indented' => \&Mod::HTML::indented,
   'endindented' => \&Mod::HTML::indented,
   'maxim' => \&Mod::HTML::indented,

   'stop' => \&Stop,
  );

%comment_table = 
  (
   'comment' => \&comment,
   'endcomment' => \&comment,
   'DEFAULT' => sub { '' },
  );

%escape = 
  ( 'X' => \&Mod::HTML::T_index,
    'V' => \&Mod::HTML::T_var,
    'I' => \&Mod::HTML::T_italics,
    'C' => \&Mod::HTML::T_code,
    'F' => \&Mod::HTML::T_function,
    'B' => \&Mod::HTML::T_bold,
    'N' => \&Mod::HTML::T_footnote,
    'T' => \&Mod::HTML::T_tag,
    'A' => \&T_href,
    'W' => \&T_wikiword,
  );

%HTML_escape = ('<' => 'lt', '>' => 'gt',
                '"' => 'quot', '&' => 'amp',
               );

################################################################

sub extension () { 'html' };

sub html_escape {
  my ($self, $text) = @_;
  $text =~ s/([<>"&])/&$HTML_escape{$1};/g;
  $text;
}

sub init {
  my $self = shift;
  my @SECTIONS = qw(section subsection subsubsection);
  $self->{section_order} = \@SECTIONS;

  $self->{index} = $self->{footnote} = 1;
  $self->{footnotes} = [];
  "";
}

sub fin {
  my $self = shift;

  my $fin = '';
  if (@{$self->{footnotes}}) {
    $fin .= "\n<hr>\n";
    $fin .= join "\n\n", @{$self->{footnotes}};
    $fin .= "\n\n\n\n";
  }
  return $fin;
}

sub prose {
  my ($self, $text, @args) = @_;
  $text = $self->expand_escapes($text);
#  $text = $self->html_escape($text);
  return "<P>$text</P>\n\n";
}

sub whitespace {
  my ($self, $text, @args) = @_;
  return;
}

sub program {
  my ($self, $text, @args) = @_;
  my $true_text = $text;
  my @marked = map 0+/^\*/, split /^/, $true_text;
  $true_text =~ s/^\t/        /mg;
  $true_text =~ s/^\*/ /mg;
  my ($prefix) = ($true_text =~  
                  m{\A(\ +).*      # First line with its prefix
                    (?:\n          # first line's newline
                     (?:\1.*\n     # subsequent lines, with same prefix
                       |\s*\n)*    #   or perhaps they're just empty
                     (?:\1.*\n?    # final line, possibly without trailer
                       |\s*$)      #   or perhaps it's just empty
                    )?             # there might be only one line
                    \z}x);

  # Trim off prefix
  my @lines = split /^/, $true_text;
  chomp @lines;
  for (@lines) {
    s/^$prefix/        /;
    $_ = $self->html_escape($_);
  }
  

  for my $i (0 .. $#lines) {
    $lines[$i] = "<font color=purple><b>$lines[$i]</b></font>" if $marked[$i];
  }

  join "\n", "<pre>", @lines, "</pre>", "";
}

sub listing {
  my ($self, $tag, $text, $progname, @args) = @_;
  qq{<p><font size="-1"><a href="$self->{progdir}/$progname">Download code for <tt>$progname</tt></a></font></p>};
}

sub inline {
  my ($self, $tag, $text, $progname, @args) = @_;
  my $PROGDIR = $self->{progdir};
  my $fh;
  unless (open $fh, "< $PROGDIR/$progname") {
    $self->warning("Couldn't inline program '$progname': $!; skipping");
    return "--- $progname ??? ---\n";
  }
  my @code = <$fh>;
  chomp @code;
  for (@code) {
    s/^/        /;
    $_ = $self->html_escape($_);
  }
  return join "\n", "<pre>", @code, "</pre>", "\n";
}

sub section {
  my ($self, $tag, $text, @args) = @_;
  my @SECTIONS = @{$self->{section_order}};
  my $nums = $self->{current_section_numbers};
  my $level;

  for (my $i=0; $i<@SECTIONS; $i++) {
    if ($tag eq $SECTIONS[$i]) {
      $level = $i; last;
    }
  }
  if ($level) {
    $#$nums = $level;
    $nums->[$level]++;
  }
  my $n = join '.', @$nums;

  $text = $self->expand_escapes($text);
  {
    my $toc = $self->{tocFH};
    print $toc "$n $text\n";
  }
  { my $l = "$n: $text";
    $l .= " " x (78 - length $l) ;
    print STDERR "$l\r";
  }
  my $h = $level + 1;
  $text = qq{<a name="section-$n">$text</a>} if $level;
  qq{<h$h>$text</h$h>\n\n};
}

sub list {
  my ($self, $tag, $text, @args) = @_;
  my $listcontext = $self->{listcontext};
  $tag =~ s/(ed)?list$//;
  my $end = ($tag =~ s/^end//) ? '/' : '';
  my $ol = $tag =~ /bullet/ ? 'ul' : 'ol';

  "<$end$ol>\n";
}

sub listitem {
  my ($self, $tag, $text, @args) = @_;

  $text = $self->expand_escapes($text);
  "<li>$text\n";
}


# $prefix isn't supported yet.
sub indented {
  my ($self, $tag, $text, $prefix, @args) = @_;
  return $tag eq 'endindented' ? "</blockquote>\n" : "<blockquote>\n";
#  $text = $self->expand_escapes($text);
#  return "<blockquote>\n$text\n</blockquote>\n\n";
}

sub Stop {
  return '', Stop => 1;
}

################################################################

# NOT FINISHED
sub T_index {
  my ($self, $char, $text) = @_;
  my $n = $self->{index}++;
  my (@items) = split /\|/, $text;
  s/\*\*\*OR/||/g for @items;   # Unpleasant HACK
  my $flags = (@items == 1) ? '' : pop @items;
  return '' if $flags =~ /[i()]/;
  if ($flags =~ s/d//) {
    $items[0] =~  s/(.*)/I<$1>/;
    $flags .= 'I';
  } elsif ($flags =~ s/f//) {
    $items[0] =~  s/(.*)/C<$1>/;
    $flags .= 'B';
  } elsif ($flags =~ s/r//) {
    $items[0] =~  s/(.*)/I<$1>/;
  }
  my $main = shift @items;
  #  $self->warning("Index item `$main' in style [$flags]");
  qq{<a name="index-$n">} . $self->expand_escapes($main) . "</a>";
}

sub T_var {
  my ($self, $char, $text) = @_;
  "<var>$text</var>";
}

sub T_math {
  my ($self, $char, $text) = @_;
  my $math = $self->{math};
  my $repl = $math->{$text};
  return $self->html_escape($repl) if defined $repl;
  unless (fileno $self->{badmath}) {
    $self->warning("Bad math escape `M<$text>'");
    open my $bad, ">> badmath"
      or die "Couldn't open `badmath': $!; aborting";
    $self->{badmath} = $bad;
  }
  my $bad = $self->{badmath};
  print $bad "$text\n";
  return $self->html_escape($text);
}

sub T_italics {
  my ($self, $char, $text) = @_;
  $text = $self->expand_escapes($text);
  "<i>$text</i>";
}

sub T_bold {
  my ($self, $char, $text) = @_;
  $text = $self->expand_escapes($text);
  "<b>$text</b>";
}

sub T_code {
  my ($self, $char, $text) = @_;
  $text =~ s/\\([\\<>])/$1/g;
  $text = $self->html_escape($text);
  "<tt>$text</tt>";
}

sub T_function {
  my ($self, $char, $text) = @_;
  $text = $self->expand_escapes($text);
  "<tt>$text()</tt>";
}

sub T_footnote {
  my ($self, $char, $text) = @_;
  my $n = $self->{footnote}++;
  $text = $self->expand_escapes($text);
  push @{$self->{footnotes}}, 
    qq{<p><a name="footnote-$n">[$n] $text</a></p>};
  return qq{ <a href="#footnote-$n">[$n]</a>};
}

sub T_tag {
  my ($self, $char, $text) = @_;
  "<tt>&lt;$text&gt;</tt>";
}

sub T_href {
  my ($self, $char, $text) = @_;
  my (@items) = split /\|/, $text; 
  my $url = pop @items;
  $text = join "", @items;
  qq{<a href="$url">$text</a>};
}

sub T_wikiword {
  my ($self, $char, $text) = @_;
  if (Wiki::Page::exists($text)) {
    qq{<a href="/wiki/$text/">$text</a>};
  } else {
    qq{$text<a href="/wiki/$text/?a=Create">?</a>};
  }
}

1;

__DATA__
