package Wiki::Request;
use Carp;
use strict;
use Wiki::Action;

sub new {
  my ($pack, $client, $type, $page, $data) = @_;
  croak('Usage: Wiki::Request::new(client, type, page, [data])')
    unless defined $client && defined $type && defined $page;
  $data ||= {};
  bless {C => $client, T => $type, P => $page, D => $data} => $pack;
}

sub fake {
  my $pack = shift;
  bless {D => { SERVER_NAME => 'perl.plover.com',
                SERVER_PORT => 80,
                SCRIPT_NAME => '/wiki',
              },
        } => $pack;
}

sub assign_semantics {
  my ($self, $semantics) = @_;
  $self->{TABLE} = $semantics;
}

sub execute {
  my ($self) = @_;
  my $table = $self->{TABLE};
  croak "Request has no defined semantics"
    unless defined $table;

  my $action = $table->{$self->type} || $table->{""};
  if (defined $action) {
    $action->($self);
  }
}

sub type { $_[0]{T} }
sub page { $_[0]{P} }
sub data { $_[0]{D} }
sub sock { $_[0]{C} }
sub user { my $u = $_[0]{D}{REMOTE_USER};
           $u =~ tr/A-Za-z0-9://cd;
           $u;
         }

sub set_type { $_[0]{T} = $_[1] }
sub set_data {
  my $self = shift;
  while (@_) {
    my $k = shift;
    my $v = shift;
    $self->{D}{$k} = $v;
  }
}

sub log {
  my $self = shift;
  my $user = $self->user || '-';
  print STDERR join " ", $self->type, $self->page, $self->user || '-', "\n";
}

sub send_response {
  my $self = shift;
  my $client = $self->{C};
  $client->wrns(join "", @_);
}

sub send_to_server {
  my $self = shift;
  my $server = $self->sock;
  $server->send_request($self);
}

sub action {
  my $self = shift;
  Wiki::Action::action($self->type);
}

1;

